// TouchPadBlocker.cpp

#include <syslog.h>

#include <AppDefs.h>
#include <Message.h>

#include "TouchPadBlocker.h"

#define LOGGING 0

// instantiate_input_filter
BInputServerFilter*
instantiate_input_filter()
{
	return (new TouchPadBlocker());
}

// constructor
TouchPadBlocker::TouchPadBlocker()
	: BInputServerFilter(),
	  fLastKeyDownTime(0),
	  fAllowClicks(true),
	  fClickDropped(false)
{
}

// destructor
TouchPadBlocker::~TouchPadBlocker()
{
}

// Filter
filter_result
TouchPadBlocker::Filter(BMessage* message, BList* outList)
{
	filter_result result = B_DISPATCH_MESSAGE;

	switch (message->what) {
		case B_KEY_DOWN:
//		case B_KEY_UP:
			// the last time when this event happened
			// is always remembered, key repeats don't
			// turn off mouse clicks though, this improves
			// for example the behaviour of holding down
			// space in graphics apps and then panning the
			// canvas
			if (!message->HasInt32("be:key_repeat")) {
				if (fAllowClicks) {
#if LOGGING
syslog(LOG_NOTICE, "key down - disabling clicks\n");
#endif
					fAllowClicks = false;
				}
			}
			fLastKeyDownTime = system_time();
			break;

		case B_MOUSE_MOVED:
			// the theory is that unwanted clicks are not
			// followed by a mouse move, and if they do, we better
			// undo skipping the event
			if (!fAllowClicks) {
#if LOGGING
syslog(LOG_NOTICE, "mouse moved - reenabling clicks\n");
#endif
				fAllowClicks = true;
			}
			if (fClickDropped) {
				// turn this into a B_MOUSE_DOWN message
				*message = fDroppedMouseDownMessage;
				fClickDropped = false;
			}
			break;

		case B_MOUSE_DOWN: {
			// double click events are certainly wanted and
			// therefor reestablish mouse down events
			int32 clicks;
			if (message->FindInt32("clicks", &clicks) >= B_OK && clicks > 1) {
#if LOGGING
syslog(LOG_NOTICE, "double click - reenabling clicks\n");
#endif
				fAllowClicks = true;
			}
			// if we decide to eat a mouse down event, we
			// store it for when we find out later that this
			// was maybe unwanted
			if (!fAllowClicks) {
				if (system_time() - fLastKeyDownTime < 1000000) {
#if LOGGING
syslog(LOG_NOTICE, "skipping mouse down\n");
#endif
					result = B_SKIP_MESSAGE;
					fClickDropped = true;
					fDroppedMouseDownMessage = *message;
				} else
					fAllowClicks = true;
			}
			break;
		}

		case B_MOUSE_UP:
			// when a previous mouse down event has been eaten,
			// we simply eat the corresponding mouse up event as well
			if (fClickDropped) {
#if LOGGING
syslog(LOG_NOTICE, "skipping mouse up\n");
#endif
				result = B_SKIP_MESSAGE;
				fClickDropped = false;
			}
			break;

		default:
			break;
	}
	return result;
}
